"""Generated message classes for telcoautomation version v1.

APIs to automate management of cloud infrastructure for network functions.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'telcoautomation'


class ApplyDeploymentRequest(_messages.Message):
  r"""Request object for `ApplyDeployment`. The resources in given deployment
  gets applied to Orchestration Cluster. A new revision is created when a
  deployment is applied.
  """



class ApplyHydratedDeploymentRequest(_messages.Message):
  r"""Request for applying a hydrated deployment."""


class ApproveBlueprintRequest(_messages.Message):
  r"""Request object for `ApproveBlueprint`."""


class Blueprint(_messages.Message):
  r"""A Blueprint contains a collection of kubernetes resources in the form of
  YAML files. The file contents of a blueprint are collectively known as
  package. A blueprint can be a) imported from TNA's public catalog b)
  modified as per a user's need c) proposed and approved. On approval, a
  revision of blueprint is created which can be used to create a deployment on
  Orchestration or Workload Cluster.

  Enums:
    ApprovalStateValueValuesEnum: Output only. Approval state of the blueprint
      (DRAFT, PROPOSED, APPROVED)
    DeploymentLevelValueValuesEnum: Output only. DeploymentLevel of a
      blueprint signifies where the blueprint will be applied. e.g.
      [HYDRATION, SINGLE_DEPLOYMENT, MULTI_DEPLOYMENT]

  Messages:
    LabelsValue: Optional. Labels are key-value attributes that can be set on
      a blueprint resource by the user.

  Fields:
    approvalState: Output only. Approval state of the blueprint (DRAFT,
      PROPOSED, APPROVED)
    createTime: Output only. Blueprint creation time.
    deploymentLevel: Output only. DeploymentLevel of a blueprint signifies
      where the blueprint will be applied. e.g. [HYDRATION, SINGLE_DEPLOYMENT,
      MULTI_DEPLOYMENT]
    displayName: Optional. Human readable name of a Blueprint.
    files: Optional. Files present in a blueprint. When invoking
      UpdateBlueprint API, only the modified files should be included in this.
      Files that are not included in the update of a blueprint will not be
      changed.
    labels: Optional. Labels are key-value attributes that can be set on a
      blueprint resource by the user.
    name: The name of the blueprint. If unspecified, the name will be
      autogenerated from server side. Name of the blueprint must not contain
      `@` character.
    repository: Output only. Name of the repository where the blueprint files
      are stored.
    revisionCreateTime: Output only. The timestamp that the revision was
      created.
    revisionId: Output only. Immutable. The revision ID of the blueprint. A
      new revision is committed whenever a blueprint is approved.
    rollbackSupport: Output only. Indicates if the deployment created from
      this blueprint can be rolled back.
    sourceBlueprint: Required. Immutable. The public blueprint ID from which
      this blueprint was created.
    sourceProvider: Output only. Source provider is the author of a public
      blueprint, from which this blueprint is created.
    updateTime: Output only. The timestamp when the blueprint was updated.
  """

  class ApprovalStateValueValuesEnum(_messages.Enum):
    r"""Output only. Approval state of the blueprint (DRAFT, PROPOSED,
    APPROVED)

    Values:
      APPROVAL_STATE_UNSPECIFIED: Unspecified state.
      DRAFT: A blueprint starts in DRAFT state once it is created. All edits
        are made to the blueprint in DRAFT state.
      PROPOSED: When the edits are ready for review, blueprint can be proposed
        and moves to PROPOSED state. Edits cannot be made to a blueprint in
        PROPOSED state.
      APPROVED: When a proposed blueprint is approved, it moves to APPROVED
        state. A new revision is committed. The latest committed revision can
        be used to create a deployment on Orchestration or Workload Cluster.
        Edits to an APPROVED blueprint changes its state back to DRAFT. The
        last committed revision of a blueprint represents its latest APPROVED
        state.
    """
    APPROVAL_STATE_UNSPECIFIED = 0
    DRAFT = 1
    PROPOSED = 2
    APPROVED = 3

  class DeploymentLevelValueValuesEnum(_messages.Enum):
    r"""Output only. DeploymentLevel of a blueprint signifies where the
    blueprint will be applied. e.g. [HYDRATION, SINGLE_DEPLOYMENT,
    MULTI_DEPLOYMENT]

    Values:
      DEPLOYMENT_LEVEL_UNSPECIFIED: Default unspecified deployment level.
      HYDRATION: Blueprints at HYDRATION level cannot be used to create a
        Deployment (A user cannot manually initiate deployment of these
        blueprints on orchestration or workload cluster). These blueprints
        stay in a user's private catalog and are configured and deployed by
        TNA automation.
      SINGLE_DEPLOYMENT: Blueprints at SINGLE_DEPLOYMENT level can be a)
        Modified in private catalog. b) Used to create a deployment on
        orchestration cluster by the user, once approved.
      MULTI_DEPLOYMENT: Blueprints at MULTI_DEPLOYMENT level can be a)
        Modified in private catalog. b) Used to create a deployment on
        orchestration cluster which will create further hydrated deployments.
      WORKLOAD_CLUSTER_DEPLOYMENT: Blueprints at WORKLOAD_CLUSTER_DEPLOYMENT
        level can be a) Modified in private catalog. b) Used to create a
        deployment on workload cluster by the user, once approved.
    """
    DEPLOYMENT_LEVEL_UNSPECIFIED = 0
    HYDRATION = 1
    SINGLE_DEPLOYMENT = 2
    MULTI_DEPLOYMENT = 3
    WORKLOAD_CLUSTER_DEPLOYMENT = 4

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. Labels are key-value attributes that can be set on a
    blueprint resource by the user.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  approvalState = _messages.EnumField('ApprovalStateValueValuesEnum', 1)
  createTime = _messages.StringField(2)
  deploymentLevel = _messages.EnumField('DeploymentLevelValueValuesEnum', 3)
  displayName = _messages.StringField(4)
  files = _messages.MessageField('File', 5, repeated=True)
  labels = _messages.MessageField('LabelsValue', 6)
  name = _messages.StringField(7)
  repository = _messages.StringField(8)
  revisionCreateTime = _messages.StringField(9)
  revisionId = _messages.StringField(10)
  rollbackSupport = _messages.BooleanField(11)
  sourceBlueprint = _messages.StringField(12)
  sourceProvider = _messages.StringField(13)
  updateTime = _messages.StringField(14)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class CidrBlock(_messages.Message):
  r"""CidrBlock contains an optional name and one CIDR block.

  Fields:
    cidrBlock: Optional. cidr_block must be specified in CIDR notation when
      using master_authorized_networks_config. Currently, the user could still
      use the deprecated man_block field, so this field is currently optional,
      but will be required in the future.
    displayName: Optional. display_name is an optional field for users to
      identify CIDR blocks.
  """

  cidrBlock = _messages.StringField(1)
  displayName = _messages.StringField(2)


class ComputeDeploymentStatusResponse(_messages.Message):
  r"""Response object for `ComputeDeploymentStatus`.

  Enums:
    AggregatedStatusValueValuesEnum: Output only. Aggregated status of a
      deployment.

  Fields:
    aggregatedStatus: Output only. Aggregated status of a deployment.
    name: The name of the deployment.
    resourceStatuses: Output only. Resource level status details in
      deployments.
  """

  class AggregatedStatusValueValuesEnum(_messages.Enum):
    r"""Output only. Aggregated status of a deployment.

    Values:
      STATUS_UNSPECIFIED: Unknown state.
      STATUS_IN_PROGRESS: Under progress.
      STATUS_ACTIVE: Running and ready to serve traffic.
      STATUS_FAILED: Failed or stalled.
      STATUS_DELETING: Delete in progress.
      STATUS_DELETED: Deleted deployment.
      STATUS_PEERING: NFDeploy specific status. Peering in progress.
      STATUS_NOT_APPLICABLE: K8s objects such as NetworkAttachmentDefinition
        don't have a defined status.
    """
    STATUS_UNSPECIFIED = 0
    STATUS_IN_PROGRESS = 1
    STATUS_ACTIVE = 2
    STATUS_FAILED = 3
    STATUS_DELETING = 4
    STATUS_DELETED = 5
    STATUS_PEERING = 6
    STATUS_NOT_APPLICABLE = 7

  aggregatedStatus = _messages.EnumField('AggregatedStatusValueValuesEnum', 1)
  name = _messages.StringField(2)
  resourceStatuses = _messages.MessageField('ResourceStatus', 3, repeated=True)


class Deployment(_messages.Message):
  r"""Deployment contains a collection of YAML files (This collection is also
  known as package) that can to applied on an orchestration cluster (GKE
  cluster with TNA addons) or a workload cluster.

  Enums:
    DeploymentLevelValueValuesEnum: Output only. Attributes to where the
      deployment can inflict changes. The value can only be
      [SINGLE_DEPLOYMENT, MULTI_DEPLOYMENT].
    StateValueValuesEnum: Output only. State of the deployment (DRAFT,
      APPLIED, DELETING).

  Messages:
    LabelsValue: Optional. Labels are key-value attributes that can be set on
      a deployment resource by the user.

  Fields:
    createTime: Output only. Deployment creation time.
    deploymentLevel: Output only. Attributes to where the deployment can
      inflict changes. The value can only be [SINGLE_DEPLOYMENT,
      MULTI_DEPLOYMENT].
    displayName: Optional. Human readable name of a Deployment.
    files: Optional. Files present in a deployment. When invoking
      UpdateDeployment API, only the modified files should be included in
      this. Files that are not included in the update of a deployment will not
      be changed.
    labels: Optional. Labels are key-value attributes that can be set on a
      deployment resource by the user.
    name: The name of the deployment.
    repository: Output only. Name of the repository where the deployment
      package files are stored.
    revisionCreateTime: Output only. The timestamp that the revision was
      created.
    revisionId: Output only. Immutable. The revision ID of the deployment. A
      new revision is committed whenever a change in deployment is applied.
    rollbackSupport: Output only. Indicates if the deployment can be rolled
      back, exported from public blueprint.
    sourceBlueprintRevision: Required. The blueprint revision from which this
      deployment was created.
    sourceProvider: Output only. Source provider is the author of a public
      blueprint, from which this deployment is created.
    state: Output only. State of the deployment (DRAFT, APPLIED, DELETING).
    updateTime: Output only. The timestamp when the deployment was updated.
    workloadCluster: Optional. Immutable. The WorkloadCluster on which to
      create the Deployment. This field should only be passed when the
      deployment_level of the source blueprint specifies deployments on
      workload clusters e.g. WORKLOAD_CLUSTER_DEPLOYMENT.
  """

  class DeploymentLevelValueValuesEnum(_messages.Enum):
    r"""Output only. Attributes to where the deployment can inflict changes.
    The value can only be [SINGLE_DEPLOYMENT, MULTI_DEPLOYMENT].

    Values:
      DEPLOYMENT_LEVEL_UNSPECIFIED: Default unspecified deployment level.
      HYDRATION: Blueprints at HYDRATION level cannot be used to create a
        Deployment (A user cannot manually initiate deployment of these
        blueprints on orchestration or workload cluster). These blueprints
        stay in a user's private catalog and are configured and deployed by
        TNA automation.
      SINGLE_DEPLOYMENT: Blueprints at SINGLE_DEPLOYMENT level can be a)
        Modified in private catalog. b) Used to create a deployment on
        orchestration cluster by the user, once approved.
      MULTI_DEPLOYMENT: Blueprints at MULTI_DEPLOYMENT level can be a)
        Modified in private catalog. b) Used to create a deployment on
        orchestration cluster which will create further hydrated deployments.
      WORKLOAD_CLUSTER_DEPLOYMENT: Blueprints at WORKLOAD_CLUSTER_DEPLOYMENT
        level can be a) Modified in private catalog. b) Used to create a
        deployment on workload cluster by the user, once approved.
    """
    DEPLOYMENT_LEVEL_UNSPECIFIED = 0
    HYDRATION = 1
    SINGLE_DEPLOYMENT = 2
    MULTI_DEPLOYMENT = 3
    WORKLOAD_CLUSTER_DEPLOYMENT = 4

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. State of the deployment (DRAFT, APPLIED, DELETING).

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      DRAFT: A deployment starts in DRAFT state. All edits are made in DRAFT
        state. A deployment opened for editing after applying will be in draft
        state, while its prevision revision will be its current applied
        version.
      APPLIED: This state means that the contents (YAML files containing
        kubernetes resources) of the deployment have been applied to an
        Orchestration or Workload Cluster. A revision is created when a
        deployment is applied. This revision will represent the latest view of
        what is applied on the cluster until the deployment is modified and
        applied again, which will create a new revision.
      DELETING: A deployment in DELETING state has been marked for deletion.
        Its deletion status can be queried using `ComputeDeploymentStatus`
        API. No updates are allowed to a deployment in DELETING state.
    """
    STATE_UNSPECIFIED = 0
    DRAFT = 1
    APPLIED = 2
    DELETING = 3

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. Labels are key-value attributes that can be set on a
    deployment resource by the user.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  deploymentLevel = _messages.EnumField('DeploymentLevelValueValuesEnum', 2)
  displayName = _messages.StringField(3)
  files = _messages.MessageField('File', 4, repeated=True)
  labels = _messages.MessageField('LabelsValue', 5)
  name = _messages.StringField(6)
  repository = _messages.StringField(7)
  revisionCreateTime = _messages.StringField(8)
  revisionId = _messages.StringField(9)
  rollbackSupport = _messages.BooleanField(10)
  sourceBlueprintRevision = _messages.StringField(11)
  sourceProvider = _messages.StringField(12)
  state = _messages.EnumField('StateValueValuesEnum', 13)
  updateTime = _messages.StringField(14)
  workloadCluster = _messages.StringField(15)


class DiscardBlueprintChangesRequest(_messages.Message):
  r"""Request object for `DiscardBlueprintChanges`."""


class DiscardBlueprintChangesResponse(_messages.Message):
  r"""Response object for `DiscardBlueprintChanges`."""


class DiscardDeploymentChangesRequest(_messages.Message):
  r"""Request object for `DiscardDeploymentChanges`."""


class DiscardDeploymentChangesResponse(_messages.Message):
  r"""Response object for `DiscardDeploymentChanges`."""


class EdgeSlm(_messages.Message):
  r"""EdgeSlm represents an SLM instance which manages the lifecycle of edge
  components installed on Workload clusters managed by an Orchestration
  Cluster.

  Enums:
    StateValueValuesEnum: Output only. State of the EdgeSlm resource.
    WorkloadClusterTypeValueValuesEnum: Optional. Type of workload cluster for
      which an EdgeSLM resource is created.

  Messages:
    LabelsValue: Optional. Labels as key value pairs. The key and value should
      contain characters which are UTF-8 compliant and less than 50
      characters.

  Fields:
    createTime: Output only. [Output only] Create time stamp.
    labels: Optional. Labels as key value pairs. The key and value should
      contain characters which are UTF-8 compliant and less than 50
      characters.
    name: Name of the EdgeSlm resource.
    orchestrationCluster: Immutable. Reference to the orchestration cluster on
      which templates for this resources will be applied. This should be of
      format projects/{project}/locations/{location}/orchestrationClusters/{or
      chestration_cluster}.
    state: Output only. State of the EdgeSlm resource.
    tnaVersion: Output only. Provides the active TNA version for this
      resource.
    updateTime: Output only. [Output only] Update time stamp.
    workloadClusterType: Optional. Type of workload cluster for which an
      EdgeSLM resource is created.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. State of the EdgeSlm resource.

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      CREATING: EdgeSlm is being created.
      ACTIVE: EdgeSlm has been created and is ready for use.
      DELETING: EdgeSlm is being deleted.
      FAILED: EdgeSlm encountered an error and is in an indeterministic state.
        User can still initiate a delete operation on this state.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    ACTIVE = 2
    DELETING = 3
    FAILED = 4

  class WorkloadClusterTypeValueValuesEnum(_messages.Enum):
    r"""Optional. Type of workload cluster for which an EdgeSLM resource is
    created.

    Values:
      WORKLOAD_CLUSTER_TYPE_UNSPECIFIED: Unspecified workload cluster.
      GDCE: Workload cluster is a GDCE cluster.
      GKE: Workload cluster is a GKE cluster.
    """
    WORKLOAD_CLUSTER_TYPE_UNSPECIFIED = 0
    GDCE = 1
    GKE = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. Labels as key value pairs. The key and value should contain
    characters which are UTF-8 compliant and less than 50 characters.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  name = _messages.StringField(3)
  orchestrationCluster = _messages.StringField(4)
  state = _messages.EnumField('StateValueValuesEnum', 5)
  tnaVersion = _messages.StringField(6)
  updateTime = _messages.StringField(7)
  workloadClusterType = _messages.EnumField('WorkloadClusterTypeValueValuesEnum', 8)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class File(_messages.Message):
  r"""File represents a yaml file present in a blueprint's package.

  Fields:
    content: Optional. The contents of a file in string format.
    deleted: Optional. Signifies whether a file is marked for deletion.
    editable: Optional. Indicates whether changes are allowed to a file. If
      the field is not set, the file cannot be edited.
    path: Required. Path of the file in package. e.g. `gdce/v1/cluster.yaml`
  """

  content = _messages.StringField(1)
  deleted = _messages.BooleanField(2)
  editable = _messages.BooleanField(3)
  path = _messages.StringField(4)


class FullManagementConfig(_messages.Message):
  r"""Configuration of the full (Autopilot) cluster management

  Fields:
    clusterCidrBlock: Optional. The IP address range for the cluster pod IPs.
      Set to blank to have a range chosen with the default size. Set to
      /netmask (e.g. /14) to have a range chosen with a specific netmask. Set
      to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private
      networks (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a
      specific range to use.
    clusterNamedRange: Optional. The name of the existing secondary range in
      the cluster's subnetwork to use for pod IP addresses. Alternatively,
      cluster_cidr_block can be used to automatically create a GKE-managed
      one.
    masterAuthorizedNetworksConfig: Optional. Master Authorized Network that
      supports multiple CIDR blocks. Allows access to the k8s master from
      multiple blocks. It cannot be set at the same time with the field
      man_block.
    masterIpv4CidrBlock: Optional. The /28 network that the masters will use.
    network: Optional. Name of the VPC Network to put the GKE cluster and
      nodes in. The VPC will be created if it doesn't exist.
    servicesCidrBlock: Optional. The IP address range for the cluster service
      IPs. Set to blank to have a range chosen with the default size. Set to
      /netmask (e.g. /14) to have a range chosen with a specific netmask. Set
      to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private
      networks (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a
      specific range to use.
    servicesNamedRange: Optional. The name of the existing secondary range in
      the cluster's subnetwork to use for service ClusterIPs. Alternatively,
      services_cidr_block can be used to automatically create a GKE-managed
      one.
    subnet: Optional. Specifies the subnet that the interface will be part of.
      Network key must be specified and the subnet must be a subnetwork of the
      specified network.
  """

  clusterCidrBlock = _messages.StringField(1)
  clusterNamedRange = _messages.StringField(2)
  masterAuthorizedNetworksConfig = _messages.MessageField('MasterAuthorizedNetworksConfig', 3)
  masterIpv4CidrBlock = _messages.StringField(4)
  network = _messages.StringField(5)
  servicesCidrBlock = _messages.StringField(6)
  servicesNamedRange = _messages.StringField(7)
  subnet = _messages.StringField(8)


class HydratedDeployment(_messages.Message):
  r"""A collection of kubernetes yaml files which are deployed on a Workload
  Cluster. Hydrated Deployments are created by TNA intent based automation.

  Enums:
    StateValueValuesEnum: Output only. State of the hydrated deployment
      (DRAFT, APPLIED).

  Fields:
    files: Optional. File contents of a hydrated deployment. When invoking
      UpdateHydratedBlueprint API, only the modified files should be included
      in this. Files that are not included in the update of a hydrated
      deployment will not be changed.
    name: Output only. The name of the hydrated deployment.
    state: Output only. State of the hydrated deployment (DRAFT, APPLIED).
    workloadCluster: Output only. WorkloadCluster identifies which workload
      cluster will the hydrated deployment will be deployed on.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. State of the hydrated deployment (DRAFT, APPLIED).

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      DRAFT: A hydrated deployment starts in DRAFT state. All edits are made
        in DRAFT state.
      APPLIED: When the edit is applied, the hydrated deployment moves to
        APPLIED state. No changes can be made once a hydrated deployment is
        applied.
    """
    STATE_UNSPECIFIED = 0
    DRAFT = 1
    APPLIED = 2

  files = _messages.MessageField('File', 1, repeated=True)
  name = _messages.StringField(2)
  state = _messages.EnumField('StateValueValuesEnum', 3)
  workloadCluster = _messages.StringField(4)


class HydrationStatus(_messages.Message):
  r"""Hydration status.

  Fields:
    siteVersion: Output only. SiteVersion Hydration is targeting.
    status: Output only. Status.
  """

  siteVersion = _messages.MessageField('SiteVersion', 1)
  status = _messages.StringField(2)


class ListBlueprintRevisionsResponse(_messages.Message):
  r"""Response object for `ListBlueprintRevisions`.

  Fields:
    blueprints: The revisions of the blueprint.
    nextPageToken: A token that can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
  """

  blueprints = _messages.MessageField('Blueprint', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListBlueprintsResponse(_messages.Message):
  r"""Response object for `ListBlueprints`.

  Fields:
    blueprints: The list of requested blueprints.
    nextPageToken: A token that can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
  """

  blueprints = _messages.MessageField('Blueprint', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListDeploymentRevisionsResponse(_messages.Message):
  r"""List of deployment revisions for a given deployment.

  Fields:
    deployments: The revisions of the deployment.
    nextPageToken: A token that can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
  """

  deployments = _messages.MessageField('Deployment', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListDeploymentsResponse(_messages.Message):
  r"""Response object for `ListDeployments`.

  Fields:
    deployments: The list of requested deployments.
    nextPageToken: A token that can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
  """

  deployments = _messages.MessageField('Deployment', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListEdgeSlmsResponse(_messages.Message):
  r"""Message for response to listing EdgeSlms.

  Fields:
    edgeSlms: The list of EdgeSlm
    nextPageToken: A token identifying a page of results the server should
      return.
    unreachable: Locations that could not be reached.
  """

  edgeSlms = _messages.MessageField('EdgeSlm', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListHydratedDeploymentsResponse(_messages.Message):
  r"""Response object for `ListHydratedDeployments`.

  Fields:
    hydratedDeployments: The list of hydrated deployments.
    nextPageToken: A token that can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
  """

  hydratedDeployments = _messages.MessageField('HydratedDeployment', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListOrchestrationClustersResponse(_messages.Message):
  r"""Message for response to listing OrchestrationClusters.

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    orchestrationClusters: The list of OrchestrationCluster
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  orchestrationClusters = _messages.MessageField('OrchestrationCluster', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListPublicBlueprintsResponse(_messages.Message):
  r"""Response object for `ListPublicBlueprints`.

  Fields:
    nextPageToken: Output only. A token identifying a page of results the
      server should return.
    publicBlueprints: The list of public blueprints to return.
  """

  nextPageToken = _messages.StringField(1)
  publicBlueprints = _messages.MessageField('PublicBlueprint', 2, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class ManagementConfig(_messages.Message):
  r"""Configuration of the cluster management

  Fields:
    fullManagementConfig: Configuration of the full (Autopilot) cluster
      management. Full cluster management is a preview feature.
    standardManagementConfig: Configuration of the standard (GKE) cluster
      management
  """

  fullManagementConfig = _messages.MessageField('FullManagementConfig', 1)
  standardManagementConfig = _messages.MessageField('StandardManagementConfig', 2)


class MasterAuthorizedNetworksConfig(_messages.Message):
  r"""Configuration of the Master Authorized Network that support multiple
  CIDRs

  Fields:
    cidrBlocks: Optional. cidr_blocks define up to 50 external networks that
      could access Kubernetes master through HTTPS.
  """

  cidrBlocks = _messages.MessageField('CidrBlock', 1, repeated=True)


class NFDeploySiteStatus(_messages.Message):
  r"""Per-Site Status.

  Fields:
    hydration: Output only. Hydration status.
    pendingDeletion: Output only. If true, the Site Deletion is in progress.
    site: Output only. Site id.
    workload: Output only. Workload status.
  """

  hydration = _messages.MessageField('HydrationStatus', 1)
  pendingDeletion = _messages.BooleanField(2)
  site = _messages.StringField(3)
  workload = _messages.MessageField('WorkloadStatus', 4)


class NFDeployStatus(_messages.Message):
  r"""Deployment status of NFDeploy.

  Fields:
    readyNfs: Output only. Total number of NFs targeted by this deployment
      with a Ready Condition set.
    sites: Output only. Per-Site Status.
    targetedNfs: Output only. Total number of NFs targeted by this deployment
  """

  readyNfs = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  sites = _messages.MessageField('NFDeploySiteStatus', 2, repeated=True)
  targetedNfs = _messages.IntegerField(3, variant=_messages.Variant.INT32)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have google.longrunning.Operation.error value
      with a google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class OrchestrationCluster(_messages.Message):
  r"""Orchestration cluster represents a GKE cluster with config controller
  and TNA specific components installed on it.

  Enums:
    StateValueValuesEnum: Output only. State of the Orchestration Cluster.

  Messages:
    LabelsValue: Labels as key value pairs.

  Fields:
    createTime: Output only. [Output only] Create time stamp.
    labels: Labels as key value pairs.
    managementConfig: Management configuration of the underlying GKE cluster.
    name: Name of the orchestration cluster. The name of orchestration cluster
      cannot be more than 24 characters.
    state: Output only. State of the Orchestration Cluster.
    tnaVersion: Output only. Provides the TNA version installed on the
      cluster.
    updateTime: Output only. [Output only] Update time stamp.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. State of the Orchestration Cluster.

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      CREATING: OrchestrationCluster is being created.
      ACTIVE: OrchestrationCluster has been created and is ready for use.
      DELETING: OrchestrationCluster is being deleted.
      FAILED: OrchestrationCluster encountered an error and is in an
        indeterministic state. User can still initiate a delete operation on
        this state.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    ACTIVE = 2
    DELETING = 3
    FAILED = 4

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels as key value pairs.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  managementConfig = _messages.MessageField('ManagementConfig', 3)
  name = _messages.StringField(4)
  state = _messages.EnumField('StateValueValuesEnum', 5)
  tnaVersion = _messages.StringField(6)
  updateTime = _messages.StringField(7)


class ProposeBlueprintRequest(_messages.Message):
  r"""Request object for `ProposeBlueprint`."""


class PublicBlueprint(_messages.Message):
  r"""A Blueprint contains a collection of kubernetes resources in the form of
  YAML files. The file contents of a blueprint are collectively known as
  package. Public blueprint is a TNA provided blueprint that in present in
  TNA's public catalog. A user can copy the public blueprint to their private
  catalog for further modifications.

  Enums:
    DeploymentLevelValueValuesEnum: DeploymentLevel of a blueprint signifies
      where the blueprint will be applied. e.g. [HYDRATION, SINGLE_DEPLOYMENT,
      MULTI_DEPLOYMENT]

  Fields:
    deploymentLevel: DeploymentLevel of a blueprint signifies where the
      blueprint will be applied. e.g. [HYDRATION, SINGLE_DEPLOYMENT,
      MULTI_DEPLOYMENT]
    description: The description of the public blueprint.
    displayName: The display name of the public blueprint.
    name: Name of the public blueprint.
    rollbackSupport: Output only. Indicates if the deployment created from
      this blueprint can be rolled back.
    sourceProvider: Source provider is the author of a public blueprint. e.g.
      Google, vendors
  """

  class DeploymentLevelValueValuesEnum(_messages.Enum):
    r"""DeploymentLevel of a blueprint signifies where the blueprint will be
    applied. e.g. [HYDRATION, SINGLE_DEPLOYMENT, MULTI_DEPLOYMENT]

    Values:
      DEPLOYMENT_LEVEL_UNSPECIFIED: Default unspecified deployment level.
      HYDRATION: Blueprints at HYDRATION level cannot be used to create a
        Deployment (A user cannot manually initiate deployment of these
        blueprints on orchestration or workload cluster). These blueprints
        stay in a user's private catalog and are configured and deployed by
        TNA automation.
      SINGLE_DEPLOYMENT: Blueprints at SINGLE_DEPLOYMENT level can be a)
        Modified in private catalog. b) Used to create a deployment on
        orchestration cluster by the user, once approved.
      MULTI_DEPLOYMENT: Blueprints at MULTI_DEPLOYMENT level can be a)
        Modified in private catalog. b) Used to create a deployment on
        orchestration cluster which will create further hydrated deployments.
      WORKLOAD_CLUSTER_DEPLOYMENT: Blueprints at WORKLOAD_CLUSTER_DEPLOYMENT
        level can be a) Modified in private catalog. b) Used to create a
        deployment on workload cluster by the user, once approved.
    """
    DEPLOYMENT_LEVEL_UNSPECIFIED = 0
    HYDRATION = 1
    SINGLE_DEPLOYMENT = 2
    MULTI_DEPLOYMENT = 3
    WORKLOAD_CLUSTER_DEPLOYMENT = 4

  deploymentLevel = _messages.EnumField('DeploymentLevelValueValuesEnum', 1)
  description = _messages.StringField(2)
  displayName = _messages.StringField(3)
  name = _messages.StringField(4)
  rollbackSupport = _messages.BooleanField(5)
  sourceProvider = _messages.StringField(6)


class RejectBlueprintRequest(_messages.Message):
  r"""Request object for `RejectBlueprint`."""


class RemoveDeploymentRequest(_messages.Message):
  r"""Request object for `RemoveDeployment`."""


class ResourceStatus(_messages.Message):
  r"""Status of a deployment resource.

  Enums:
    ResourceTypeValueValuesEnum: Output only. Resource type.
    StatusValueValuesEnum: Output only. Status of the resource.

  Fields:
    group: Group to which the resource belongs to.
    kind: Kind of the resource.
    name: Name of the resource.
    nfDeployStatus: Output only. Detailed status of NFDeploy.
    resourceNamespace: Namespace of the resource.
    resourceType: Output only. Resource type.
    status: Output only. Status of the resource.
    version: Version of the resource.
  """

  class ResourceTypeValueValuesEnum(_messages.Enum):
    r"""Output only. Resource type.

    Values:
      RESOURCE_TYPE_UNSPECIFIED: Unspecified resource type.
      NF_DEPLOY_RESOURCE: User specified NF Deploy CR.
      DEPLOYMENT_RESOURCE: CRs that are part of a blueprint.
    """
    RESOURCE_TYPE_UNSPECIFIED = 0
    NF_DEPLOY_RESOURCE = 1
    DEPLOYMENT_RESOURCE = 2

  class StatusValueValuesEnum(_messages.Enum):
    r"""Output only. Status of the resource.

    Values:
      STATUS_UNSPECIFIED: Unknown state.
      STATUS_IN_PROGRESS: Under progress.
      STATUS_ACTIVE: Running and ready to serve traffic.
      STATUS_FAILED: Failed or stalled.
      STATUS_DELETING: Delete in progress.
      STATUS_DELETED: Deleted deployment.
      STATUS_PEERING: NFDeploy specific status. Peering in progress.
      STATUS_NOT_APPLICABLE: K8s objects such as NetworkAttachmentDefinition
        don't have a defined status.
    """
    STATUS_UNSPECIFIED = 0
    STATUS_IN_PROGRESS = 1
    STATUS_ACTIVE = 2
    STATUS_FAILED = 3
    STATUS_DELETING = 4
    STATUS_DELETED = 5
    STATUS_PEERING = 6
    STATUS_NOT_APPLICABLE = 7

  group = _messages.StringField(1)
  kind = _messages.StringField(2)
  name = _messages.StringField(3)
  nfDeployStatus = _messages.MessageField('NFDeployStatus', 4)
  resourceNamespace = _messages.StringField(5)
  resourceType = _messages.EnumField('ResourceTypeValueValuesEnum', 6)
  status = _messages.EnumField('StatusValueValuesEnum', 7)
  version = _messages.StringField(8)


class RollbackDeploymentRequest(_messages.Message):
  r"""Request object for `RollbackDeployment`.

  Fields:
    revisionId: Required. The revision id of deployment to roll back to.
  """

  revisionId = _messages.StringField(1)


class SearchBlueprintRevisionsResponse(_messages.Message):
  r"""Response object for `SearchBlueprintRevisions`.

  Fields:
    blueprints: The list of requested blueprint revisions.
    nextPageToken: A token that can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
  """

  blueprints = _messages.MessageField('Blueprint', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class SearchDeploymentRevisionsResponse(_messages.Message):
  r"""Response object for `SearchDeploymentRevisions`.

  Fields:
    deployments: The list of requested deployment revisions.
    nextPageToken: A token that can be sent as `page_token` to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
  """

  deployments = _messages.MessageField('Deployment', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class SiteVersion(_messages.Message):
  r"""SiteVersion Hydration is targeting.

  Fields:
    nfType: Output only. NF vendor type.
    nfVendor: Output only. NF vendor.
    nfVersion: Output only. NF version.
  """

  nfType = _messages.StringField(1)
  nfVendor = _messages.StringField(2)
  nfVersion = _messages.StringField(3)


class StandardManagementConfig(_messages.Message):
  r"""Configuration of the standard (GKE) cluster management.

  Fields:
    clusterCidrBlock: Optional. The IP address range for the cluster pod IPs.
      Set to blank to have a range chosen with the default size. Set to
      /netmask (e.g. /14) to have a range chosen with a specific netmask. Set
      to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private
      networks (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a
      specific range to use.
    clusterNamedRange: Optional. The name of the existing secondary range in
      the cluster's subnetwork to use for pod IP addresses. Alternatively,
      cluster_cidr_block can be used to automatically create a GKE-managed
      one.
    masterAuthorizedNetworksConfig: Optional. Master Authorized Network that
      supports multiple CIDR blocks. Allows access to the k8s master from
      multiple blocks. It cannot be set at the same time with the field
      man_block.
    masterIpv4CidrBlock: Optional. The /28 network that the masters will use.
      It should be free within the network.
    network: Optional. Name of the VPC Network to put the GKE cluster and
      nodes in. The VPC will be created if it doesn't exist.
    servicesCidrBlock: Optional. The IP address range for the cluster service
      IPs. Set to blank to have a range chosen with the default size. Set to
      /netmask (e.g. /14) to have a range chosen with a specific netmask. Set
      to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private
      networks (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a
      specific range to use.
    servicesNamedRange: Optional. The name of the existing secondary range in
      the cluster's subnetwork to use for service ClusterIPs. Alternatively,
      services_cidr_block can be used to automatically create a GKE-managed
      one.
    subnet: Optional. Specifies the subnet that the interface will be part of.
      Network key must be specified and the subnet must be a subnetwork of the
      specified network.
  """

  clusterCidrBlock = _messages.StringField(1)
  clusterNamedRange = _messages.StringField(2)
  masterAuthorizedNetworksConfig = _messages.MessageField('MasterAuthorizedNetworksConfig', 3)
  masterIpv4CidrBlock = _messages.StringField(4)
  network = _messages.StringField(5)
  servicesCidrBlock = _messages.StringField(6)
  servicesNamedRange = _messages.StringField(7)
  subnet = _messages.StringField(8)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TelcoautomationProjectsLocationsEdgeSlmsCreateRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsEdgeSlmsCreateRequest object.

  Fields:
    edgeSlm: A EdgeSlm resource to be passed as the request body.
    edgeSlmId: Required. Id of the requesting object If auto-generating Id
      server-side, remove this field and edge_slm_id from the method_signature
      of Create RPC
    parent: Required. Value for parent.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  edgeSlm = _messages.MessageField('EdgeSlm', 1)
  edgeSlmId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class TelcoautomationProjectsLocationsEdgeSlmsDeleteRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsEdgeSlmsDeleteRequest object.

  Fields:
    name: Required. Name of the resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class TelcoautomationProjectsLocationsEdgeSlmsGetRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsEdgeSlmsGetRequest object.

  Fields:
    name: Required. Name of the resource
  """

  name = _messages.StringField(1, required=True)


class TelcoautomationProjectsLocationsEdgeSlmsListRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsEdgeSlmsListRequest object.

  Fields:
    filter: Filtering results
    orderBy: Hint for how to order the results
    pageSize: Requested page size. Server may return fewer items than
      requested. If unspecified, server will pick an appropriate default.
    pageToken: A token identifying a page of results the server should return.
    parent: Required. Parent value for ListEdgeSlmsRequest
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class TelcoautomationProjectsLocationsGetRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class TelcoautomationProjectsLocationsListRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class TelcoautomationProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class TelcoautomationProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class TelcoautomationProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class TelcoautomationProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsApproveRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsApprove
  Request object.

  Fields:
    approveBlueprintRequest: A ApproveBlueprintRequest resource to be passed
      as the request body.
    name: Required. The name of the blueprint to approve. The blueprint must
      be in Proposed state. A new revision is committed on approval.
  """

  approveBlueprintRequest = _messages.MessageField('ApproveBlueprintRequest', 1)
  name = _messages.StringField(2, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsCreateRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsCreateRequest
  object.

  Fields:
    blueprint: A Blueprint resource to be passed as the request body.
    blueprintId: Optional. The name of the blueprint.
    parent: Required. The name of parent resource. Format should be - "project
      s/{project_id}/locations/{location_name}/orchestrationClusters/{orchestr
      ation_cluster}".
  """

  blueprint = _messages.MessageField('Blueprint', 1)
  blueprintId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsDeleteRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsDeleteRequest
  object.

  Fields:
    name: Required. The name of blueprint to delete. Blueprint name should be
      in the format {blueprint_id}, if {blueprint_id}@{revision_id} is passed
      then the API throws invalid argument.
  """

  name = _messages.StringField(1, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsDiscardRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsDiscard
  Request object.

  Fields:
    discardBlueprintChangesRequest: A DiscardBlueprintChangesRequest resource
      to be passed as the request body.
    name: Required. The name of the blueprint of which changes are being
      discarded.
  """

  discardBlueprintChangesRequest = _messages.MessageField('DiscardBlueprintChangesRequest', 1)
  name = _messages.StringField(2, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsGetRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsGetRequest
  object.

  Enums:
    ViewValueValuesEnum: Optional. Defines the type of view of the blueprint.
      When field is not present BLUEPRINT_VIEW_BASIC is considered as default.

  Fields:
    name: Required. The name of the blueprint. Case 1: If the name provided in
      the request is {blueprint_id}@{revision_id}, then the revision with
      revision_id will be returned. Case 2: If the name provided in the
      request is {blueprint}, then the current state of the blueprint is
      returned.
    view: Optional. Defines the type of view of the blueprint. When field is
      not present BLUEPRINT_VIEW_BASIC is considered as default.
  """

  class ViewValueValuesEnum(_messages.Enum):
    r"""Optional. Defines the type of view of the blueprint. When field is not
    present BLUEPRINT_VIEW_BASIC is considered as default.

    Values:
      BLUEPRINT_VIEW_UNSPECIFIED: Unspecified enum value.
      BLUEPRINT_VIEW_BASIC: View which only contains metadata.
      BLUEPRINT_VIEW_FULL: View which contains metadata and files it
        encapsulates.
    """
    BLUEPRINT_VIEW_UNSPECIFIED = 0
    BLUEPRINT_VIEW_BASIC = 1
    BLUEPRINT_VIEW_FULL = 2

  name = _messages.StringField(1, required=True)
  view = _messages.EnumField('ViewValueValuesEnum', 2)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsListRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsListRequest
  object.

  Fields:
    filter: Optional. Filtering only supports equality on blueprint state. It
      should be in the form: "state = DRAFT". `OR` operator can be used to get
      response for multiple states. e.g. "state = DRAFT OR state = PROPOSED".
    pageSize: Optional. The maximum number of blueprints to return per page.
    pageToken: Optional. The page token, received from a previous
      ListBlueprints call. It can be provided to retrieve the subsequent page.
    parent: Required. The name of parent orchestration cluster resource.
      Format should be - "projects/{project_id}/locations/{location_name}/orch
      estrationClusters/{orchestration_cluster}".
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsListRevisionsRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsListRev
  isionsRequest object.

  Fields:
    name: Required. The name of the blueprint to list revisions for.
    pageSize: The maximum number of revisions to return per page.
    pageToken: The page token, received from a previous ListBlueprintRevisions
      call It can be provided to retrieve the subsequent page.
  """

  name = _messages.StringField(1, required=True)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsPatchRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsPatchRequest
  object.

  Fields:
    blueprint: A Blueprint resource to be passed as the request body.
    name: The name of the blueprint. If unspecified, the name will be
      autogenerated from server side. Name of the blueprint must not contain
      `@` character.
    updateMask: Required. Update mask is used to specify the fields to be
      overwritten in the `blueprint` resource by the update.
  """

  blueprint = _messages.MessageField('Blueprint', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsProposeRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsPropose
  Request object.

  Fields:
    name: Required. The name of the blueprint being proposed.
    proposeBlueprintRequest: A ProposeBlueprintRequest resource to be passed
      as the request body.
  """

  name = _messages.StringField(1, required=True)
  proposeBlueprintRequest = _messages.MessageField('ProposeBlueprintRequest', 2)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsRejectRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsRejectRequest
  object.

  Fields:
    name: Required. The name of the blueprint being rejected.
    rejectBlueprintRequest: A RejectBlueprintRequest resource to be passed as
      the request body.
  """

  name = _messages.StringField(1, required=True)
  rejectBlueprintRequest = _messages.MessageField('RejectBlueprintRequest', 2)


class TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsSearchRevisionsRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersBlueprintsSearchR
  evisionsRequest object.

  Fields:
    pageSize: Optional. The maximum number of blueprints revisions to return
      per page. max page size = 100, default page size = 20.
    pageToken: Optional. The page token, received from a previous search call.
      It can be provided to retrieve the subsequent page.
    parent: Required. The name of parent orchestration cluster resource.
      Format should be - "projects/{project_id}/locations/{location_name}/orch
      estrationClusters/{orchestration_cluster}".
    query: Required. Supported queries: 1. "" : Lists all revisions across all
      blueprints. 2. "latest=true" : Lists latest revisions across all
      blueprints. 3. "name={name}" : Lists all revisions of blueprint with
      name {name}. 4. "name={name} latest=true": Lists latest revision of
      blueprint with name {name}
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  query = _messages.StringField(4)


class TelcoautomationProjectsLocationsOrchestrationClustersCreateRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersCreateRequest
  object.

  Fields:
    orchestrationCluster: A OrchestrationCluster resource to be passed as the
      request body.
    orchestrationClusterId: Required. Id of the requesting object If auto-
      generating Id server-side, remove this field and
      orchestration_cluster_id from the method_signature of Create RPC
    parent: Required. Value for parent.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  orchestrationCluster = _messages.MessageField('OrchestrationCluster', 1)
  orchestrationClusterId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class TelcoautomationProjectsLocationsOrchestrationClustersDeleteRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeleteRequest
  object.

  Fields:
    name: Required. Name of the resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsApplyRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsApplyRequest
  object.

  Fields:
    applyDeploymentRequest: A ApplyDeploymentRequest resource to be passed as
      the request body.
    name: Required. The name of the deployment to apply to orchestration
      cluster.
  """

  applyDeploymentRequest = _messages.MessageField('ApplyDeploymentRequest', 1)
  name = _messages.StringField(2, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsComputeDeploymentStatusRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsComput
  eDeploymentStatusRequest object.

  Fields:
    name: Required. The name of the deployment without revisionID.
  """

  name = _messages.StringField(1, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsCreateRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsCreate
  Request object.

  Fields:
    deployment: A Deployment resource to be passed as the request body.
    deploymentId: Optional. The name of the deployment.
    parent: Required. The name of parent resource. Format should be - "project
      s/{project_id}/locations/{location_name}/orchestrationClusters/{orchestr
      ation_cluster}".
  """

  deployment = _messages.MessageField('Deployment', 1)
  deploymentId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsDiscardRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsDiscar
  dRequest object.

  Fields:
    discardDeploymentChangesRequest: A DiscardDeploymentChangesRequest
      resource to be passed as the request body.
    name: Required. The name of the deployment of which changes are being
      discarded.
  """

  discardDeploymentChangesRequest = _messages.MessageField('DiscardDeploymentChangesRequest', 1)
  name = _messages.StringField(2, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsGetRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsGetRequest
  object.

  Enums:
    ViewValueValuesEnum: Optional. Defines the type of view of the deployment.
      When field is not present VIEW_BASIC is considered as default.

  Fields:
    name: Required. The name of the deployment. Case 1: If the name provided
      in the request is {deployment_id}@{revision_id}, then the revision with
      revision_id will be returned. Case 2: If the name provided in the
      request is {deployment}, then the current state of the deployment is
      returned.
    view: Optional. Defines the type of view of the deployment. When field is
      not present VIEW_BASIC is considered as default.
  """

  class ViewValueValuesEnum(_messages.Enum):
    r"""Optional. Defines the type of view of the deployment. When field is
    not present VIEW_BASIC is considered as default.

    Values:
      DEPLOYMENT_VIEW_UNSPECIFIED: Unspecified enum value.
      DEPLOYMENT_VIEW_BASIC: View which only contains metadata.
      DEPLOYMENT_VIEW_FULL: View which contains metadata and files it
        encapsulates.
    """
    DEPLOYMENT_VIEW_UNSPECIFIED = 0
    DEPLOYMENT_VIEW_BASIC = 1
    DEPLOYMENT_VIEW_FULL = 2

  name = _messages.StringField(1, required=True)
  view = _messages.EnumField('ViewValueValuesEnum', 2)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsHydratedDeploymentsApplyRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsHydrat
  edDeploymentsApplyRequest object.

  Fields:
    applyHydratedDeploymentRequest: A ApplyHydratedDeploymentRequest resource
      to be passed as the request body.
    name: Required. The name of the hydrated deployment to apply.
  """

  applyHydratedDeploymentRequest = _messages.MessageField('ApplyHydratedDeploymentRequest', 1)
  name = _messages.StringField(2, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsHydratedDeploymentsGetRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsHydrat
  edDeploymentsGetRequest object.

  Fields:
    name: Required. Name of the hydrated deployment.
  """

  name = _messages.StringField(1, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsHydratedDeploymentsListRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsHydrat
  edDeploymentsListRequest object.

  Fields:
    pageSize: Optional. The maximum number of hydrated deployments to return.
      The service may return fewer than this value. If unspecified, at most 50
      hydrated deployments will be returned. The maximum value is 1000. Values
      above 1000 will be set to 1000.
    pageToken: Optional. The page token, received from a previous
      ListHydratedDeployments call. Provide this to retrieve the subsequent
      page.
    parent: Required. The deployment managing the hydrated deployments.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsHydratedDeploymentsPatchRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsHydrat
  edDeploymentsPatchRequest object.

  Fields:
    hydratedDeployment: A HydratedDeployment resource to be passed as the
      request body.
    name: Output only. The name of the hydrated deployment.
    updateMask: Required. The list of fields to update. Update mask supports a
      special value `*` which fully replaces (equivalent to PUT) the resource
      provided.
  """

  hydratedDeployment = _messages.MessageField('HydratedDeployment', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsListRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsListRequest
  object.

  Fields:
    filter: Optional. Filtering only supports equality on deployment state. It
      should be in the form: "state = DRAFT". `OR` operator can be used to get
      response for multiple states. e.g. "state = DRAFT OR state = APPLIED".
    pageSize: Optional. The maximum number of deployments to return per page.
    pageToken: Optional. The page token, received from a previous
      ListDeployments call. It can be provided to retrieve the subsequent
      page.
    parent: Required. The name of parent orchestration cluster resource.
      Format should be - "projects/{project_id}/locations/{location_name}/orch
      estrationClusters/{orchestration_cluster}".
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsListRevisionsRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsListRe
  visionsRequest object.

  Fields:
    name: Required. The name of the deployment to list revisions for.
    pageSize: Optional. The maximum number of revisions to return per page.
    pageToken: Optional. The page token, received from a previous
      ListDeploymentRevisions call Provide this to retrieve the subsequent
      page.
  """

  name = _messages.StringField(1, required=True)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsPatchRequest(_messages.Message):
  r"""A
  TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsPatchRequest
  object.

  Fields:
    deployment: A Deployment resource to be passed as the request body.
    name: The name of the deployment.
    updateMask: Required. Update mask is used to specify the fields to be
      overwritten in the `deployment` resource by the update.
  """

  deployment = _messages.MessageField('Deployment', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsRemoveRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsRemove
  Request object.

  Fields:
    name: Required. The name of deployment to initiate delete.
    removeDeploymentRequest: A RemoveDeploymentRequest resource to be passed
      as the request body.
  """

  name = _messages.StringField(1, required=True)
  removeDeploymentRequest = _messages.MessageField('RemoveDeploymentRequest', 2)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsRollbackRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsRollba
  ckRequest object.

  Fields:
    name: Required. Name of the deployment.
    rollbackDeploymentRequest: A RollbackDeploymentRequest resource to be
      passed as the request body.
  """

  name = _messages.StringField(1, required=True)
  rollbackDeploymentRequest = _messages.MessageField('RollbackDeploymentRequest', 2)


class TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsSearchRevisionsRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersDeploymentsSearch
  RevisionsRequest object.

  Fields:
    pageSize: Optional. The maximum number of deployment revisions to return
      per page. max page size = 100, default page size = 20.
    pageToken: Optional. The page token, received from a previous search call.
      It can be provided to retrieve the subsequent page.
    parent: Required. The name of parent orchestration cluster resource.
      Format should be - "projects/{project_id}/locations/{location_name}/orch
      estrationClusters/{orchestration_cluster}".
    query: Required. Supported queries: 1. "" : Lists all revisions across all
      deployments. 2. "latest=true" : Lists latest revisions across all
      deployments. 3. "name={name}" : Lists all revisions of deployment with
      name {name}. 4. "name={name} latest=true": Lists latest revision of
      deployment with name {name}
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  query = _messages.StringField(4)


class TelcoautomationProjectsLocationsOrchestrationClustersGetRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersGetRequest
  object.

  Fields:
    name: Required. Name of the resource
  """

  name = _messages.StringField(1, required=True)


class TelcoautomationProjectsLocationsOrchestrationClustersListRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsOrchestrationClustersListRequest
  object.

  Fields:
    filter: Filtering results.
    orderBy: Hint for how to order the results.
    pageSize: Requested page size. Server may return fewer items than
      requested. If unspecified, server will pick an appropriate default.
    pageToken: A token identifying a page of results the server should return.
    parent: Required. Parent value for ListOrchestrationClustersRequest
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class TelcoautomationProjectsLocationsPublicBlueprintsGetRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsPublicBlueprintsGetRequest object.

  Fields:
    name: Required. The name of the public blueprint.
  """

  name = _messages.StringField(1, required=True)


class TelcoautomationProjectsLocationsPublicBlueprintsListRequest(_messages.Message):
  r"""A TelcoautomationProjectsLocationsPublicBlueprintsListRequest object.

  Fields:
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. If unspecified, server will pick an appropriate default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. Parent value of public blueprint. Format should be -
      "projects/{project_id}/locations/{location_name}".
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class WorkloadStatus(_messages.Message):
  r"""Workload status.

  Fields:
    siteVersion: Output only. SiteVersion running in the workload cluster.
    status: Output only. Status.
  """

  siteVersion = _messages.MessageField('SiteVersion', 1)
  status = _messages.StringField(2)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Library of methods for manipulating virtualenv setup."""


import os

from googlecloudsdk.calliope import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.util import files
from googlecloudsdk.core.util import platforms
import six

# Python modules to install into virtual env environment
MODULES = [
    'crcmod',
    'grpcio==1.80.0',
    'pyopenssl==26.0.0',
    'google_crc32c',
    'certifi',
    ('https://github.com/googleapis/enterprise-certificate-proxy/releases/down'
     'load/v0.3.15/cryptography-46.0.7-cp38-abi3-macosx_10_9_universal2.whl'),
    'setuptools',
]

# Enable file name.
ENABLE_FILE = 'enabled'


def IsPy2():
  """Wrap six.PY2, needed because mocking six.PY2 breaks test lib things."""
  return six.PY2


def IsWindows():
  """Wrapped because mocking directly can break test lib things."""
  return platforms.OperatingSystem.IsWindows()


def VirtualEnvExists(ve_dir):
  """Returns True if Virtual Env already exists."""
  return os.path.isdir(ve_dir)


def EnableFileExists(ve_dir):
  """Returns True if enable file exists."""
  return os.path.exists('{}/{}'.format(ve_dir, ENABLE_FILE))


def CreateEnableFile(ve_dir):
  """Create enable file."""
  files.WriteFileContents('{}/{}'.format(ve_dir, ENABLE_FILE), 'enabled')


def RmEnableFile(ve_dir):
  """Remove enable file."""
  os.unlink('{}/{}'.format(ve_dir, ENABLE_FILE))


def EnableVirtualEnv(ve_dir):
  """Enable virtualenv."""
  if VirtualEnvExists(ve_dir):
    if not EnableFileExists(ve_dir):
      CreateEnableFile(ve_dir)
    log.status.Print('Virtual env enabled.')
  else:
    log.error('Virtual env does not exist at {}.'.format(ve_dir))
    raise exceptions.ExitCodeNoError(exit_code=1)




# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common flags for network connectivity commands."""


import enum

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.network_connectivity import util
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


GLOBAL_ARGUMENT = '--global'
REGION_ARGUMENT = '--region'


def AddExcludeExportRangesFlag(parser, hide_exclude_export_ranges_flag):
  """Adds the --exclude-export-ranges argument to the given parser."""
  parser.add_argument(
      '--exclude-export-ranges',
      required=False,
      type=arg_parsers.ArgList(),
      default=[],
      metavar='CIDR_RANGE',
      hidden=hide_exclude_export_ranges_flag,
      help="""Subnet IP address range(s) to hide from other VPC networks that are
        connected through Network Connectivity Center.""")


def AddIncludeExportRangesFlag(parser, hide_include_export_ranges_flag):
  """Adds the --include-export-ranges argument to the given parser."""

  parser.add_argument(
      '--include-export-ranges',
      required=False,
      type=arg_parsers.ArgList(),
      default=[],
      metavar='CIDR_RANGE',
      hidden=hide_include_export_ranges_flag,
      help="""Subnet IP address range(s) to export to other VPC networks that are
        connected through Network Connectivity Center.""",
  )


def AddUpdateIncludeExportRangesFlag(
    parser, hide_include_export_ranges_flag
):
  """Adds the --include-export-ranges argument to the update operation parser."""

  parser.add_argument(
      '--include-export-ranges',
      required=False,
      type=arg_parsers.ArgList(),
      default=None,
      metavar='CIDR_RANGE',
      hidden=hide_include_export_ranges_flag,
      help="""New include export ranges of the spoke.""",
  )


def AddUpdateExcludeExportRangesFlag(
    parser, hide_exclude_export_ranges_flag
):
  """Adds the --exclude-export-ranges argument to the update operation parser."""

  parser.add_argument(
      '--exclude-export-ranges',
      required=False,
      type=arg_parsers.ArgList(),
      default=None,
      metavar='CIDR_RANGE',
      hidden=hide_exclude_export_ranges_flag,
      help="""New exclude export ranges of the spoke.""",
  )


def GetCapacityArg(gateway_message):
  custom_mappings = {
      'CAPACITY_1_GBPS': ('1g', 'Gateway will have capacity of 1 Gbps'),
      'CAPACITY_10_GBPS': ('10g', 'Gateway will have capacity of 10 Gbps'),
  }
  if 'CAPACITY_100_GBPS' in gateway_message.CapacityValueValuesEnum.names():
    custom_mappings['CAPACITY_100_GBPS'] = (
        '100g',
        'Gateway will have capacity of 100 Gbps',
    )

  return arg_utils.ChoiceEnumMapper(
      arg_name='--capacity',
      message_enum=gateway_message.CapacityValueValuesEnum,
      custom_mappings=custom_mappings,
      help_str='Set the capacity of the gateway in Gbps.',
      required=True,
  )


def AddCapacityFlag(gateway_message, parser):
  GetCapacityArg(gateway_message).choice_arg.AddToParser(parser)


def AddIpRangeReservationsFlag(parser):
  """Adds the --ip-range-reservation argument to the given parser."""
  parser.add_argument(
      '--ip-range-reservations',
      required=True,
      type=arg_parsers.ArgList(),
      default=[],
      metavar='CIDR_RANGE',
      help="""The IP range reservation for the spoke.""",
  )


def AddLandingNetworkFlag(parser):
  """Adds the --landing-network argument to the given parser."""
  # TODO: b/233653552 - Parse this with a resource argument.
  parser.add_argument(
      '--landing-network',
      help="""The landing network for the spoke. The network must already
      exist.""",
  )


def AddAsyncFlag(parser):
  """Add the --async argument to the given parser."""
  base.ASYNC_FLAG.AddToParser(parser)


def AddHubFlag(parser):
  """Adds the --hub argument to the given parser."""
  # TODO(b/233653552) Parse this with a resource argument.
  parser.add_argument(
      '--hub',
      required=True,
      help='Hub that the spoke will attach to. The hub must already exist.')


def AddSpokeFlag(parser, help_text):
  """Adds the --spoke flag to the given parser."""
  parser.add_argument(
      '--spoke',
      required=True,
      help=help_text)


def AddSpokeEtagFlag(parser, help_text):
  """Adds the --spoke-etag flag to the given parser."""
  parser.add_argument(
      '--spoke-etag',
      required=True,
      help=help_text)


def AddGroupFlag(parser, required=False):
  """Adds the --group argument to the given parser."""
  # TODO(b/233653552) Parse this with a resource argument.
  parser.add_argument(
      '--group',
      required=required,
      hidden=False,
      help=(
          'Group that the spoke will be part of. The group must already'
          ' exist.'
      ),
  )


def AddNetworkFlag(parser):
  """Adds the --network argument to the given parser."""
  parser.add_argument(
      '--network',
      required=True,
      help="""Your VPC network that contains the peering to the Producer VPC,
      which this spoke connects to the Hub. The peering must already exist and
      be in the ACTIVE state.""")


def AddPeeringFlag(parser):
  """Adds the --peering argument to the given parser."""
  parser.add_argument(
      '--peering',
      required=True,
      help="""Peering between your network and the Producer VPC, which this
      spoke connects to the Hub. The peering must already exist and be in the
      ACTIVATE state.""")


def AddVPCNetworkFlag(parser):
  """Adds the --vpc-network argument to the given parser."""
  # TODO(b/233653552) Parse this with a resource argument.
  parser.add_argument(
      '--vpc-network',
      required=True,
      help="""VPC network that the spoke provides connectivity to.
      The resource must already exist.""")


def AddDescriptionFlag(parser, help_text):
  """Adds the --description flag to the given parser."""
  parser.add_argument(
      '--description',
      required=False,
      help=help_text)


def AddRejectionDetailsFlag(parser):
  """Adds the --details flag to the given parser."""
  parser.add_argument(
      '--details',
      required=False,
      help="""Additional details behind the rejection""")


def AddGlobalFlag(parser, hidden):
  """Add the --global argument to the given parser."""
  parser.add_argument(
      GLOBAL_ARGUMENT,
      help='Indicates that the spoke is global.',
      hidden=hidden,
      action=util.StoreGlobalAction)


def AddRegionFlag(parser, supports_region_wildcard, hidden, required):
  """Add the --region argument to the given parser."""
  region_help_text = """ \
        A Google Cloud region. To see the names of regions, see [Viewing a list of available regions](https://cloud.google.com/compute/docs/regions-zones/viewing-regions-zones#viewing_a_list_of_available_regions)."""  # pylint: disable=line-too-long
  if supports_region_wildcard:
    region_help_text += ' Use ``-`` to specify all regions.'
  parser.add_argument(
      REGION_ARGUMENT, hidden=hidden, required=required, help=region_help_text
  )


def AddRegionGroup(parser,
                   supports_region_wildcard=False,
                   hide_global_arg=False,
                   hide_region_arg=False):
  """Add a group which contains the global and region arguments to the given parser."""
  region_group = parser.add_group(required=False, mutex=True)
  AddGlobalFlag(region_group, hide_global_arg)
  AddRegionFlag(
      region_group, supports_region_wildcard, hide_region_arg, required=False
  )


def AddSpokeLocationsFlag(parser):
  """Add the --spoke-locations argument to the given parser."""
  spoke_locations_help_text = """ \
        A comma separated list of locations. The locations can be set to 'global'
        and/or Google Cloud supported regions. To see the names of regions, see
        [Viewing a list of available regions](https://cloud.google.com/compute/docs/regions-zones/viewing-regions-zones#viewing_a_list_of_available_regions)."""
  parser.add_argument(
      '--spoke-locations',
      required=False,
      help=spoke_locations_help_text,
      type=arg_parsers.ArgList(),
      default=[],
      metavar='LOCATION')


def AddViewFlag(parser):
  """Add the --view argument to the given parser."""
  view_help_text = """ \
       Enumeration to control which spoke fields are included in the response."""
  parser.add_argument(
      '--view',
      required=False,
      choices=['basic', 'detailed'],
      default='basic',
      help=view_help_text)


def AddHubResourceArg(parser, desc):
  """Add a resource argument for a hub.

  Args:
    parser: the parser for the command.
    desc: the string to describe the resource, such as 'to create'.
  """
  hub_concept_spec = concepts.ResourceSpec(
      'networkconnectivity.projects.locations.global.hubs',
      resource_name='hub',
      hubsId=HubAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)

  presentation_spec = presentation_specs.ResourcePresentationSpec(
      name='hub',
      concept_spec=hub_concept_spec,
      required=True,
      group_help='Name of the hub {}.'.format(desc))
  concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)


def AddGroupResourceArg(parser, desc):
  """Add a resource argument for a group.

  Args:
    parser: the parser for the command.
    desc: the string to describe the resource, such as 'to create'.
  """
  group_concept_spec = concepts.ResourceSpec(
      'networkconnectivity.projects.locations.global.hubs.groups',
      resource_name='group',
      api_version='v1',
      groupsId=GroupAttributeConfig(),
      hubsId=HubAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)

  presentation_spec = presentation_specs.ResourcePresentationSpec(
      name='group',
      concept_spec=group_concept_spec,
      required=True,
      group_help='Name of the group {}.'.format(desc))
  concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)


def SpokeAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='spoke', help_text='The spoke Id.')


def HubAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='hub', help_text='The hub Id.'
  )


def GroupAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='group', help_text='The group Id.'
  )


def LocationAttributeConfig(location_arguments, region_resource_command=False):
  """Get a location argument with the appropriate fallthroughs."""
  location_fallthroughs = [
      deps.ArgFallthrough(arg) for arg in location_arguments
  ]
  # If this is an attribute for a region resource, add '-' as a fallthrough
  # to default to all regions.
  if region_resource_command:
    location_fallthroughs.append(
        deps.Fallthrough(
            function=lambda: '-',
            hint='defaults to all regions if not specified'))
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      help_text='The location Id.',
      fallthroughs=location_fallthroughs)


def TransportAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='transport', help_text='The transport Id.'
  )


def RemoteProfileAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='remote_profile',
      help_text='The remote profile Id.',
  )


def GetSpokeResourceSpec(location_arguments):
  return concepts.ResourceSpec(
      'networkconnectivity.projects.locations.spokes',
      resource_name='spoke',
      spokesId=SpokeAttributeConfig(),
      locationsId=LocationAttributeConfig(location_arguments),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)


def GetRegionResourceSpec(location_arguments):
  return concepts.ResourceSpec(
      'networkconnectivity.projects.locations',
      resource_name='region',
      locationsId=LocationAttributeConfig(
          location_arguments, region_resource_command=True
      ),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False,
  )


def GetTransportResourceSpec(location_arguments):
  return concepts.ResourceSpec(
      'networkconnectivity.projects.locations.transports',
      resource_name='transport',
      transportsId=TransportAttributeConfig(),
      locationsId=LocationAttributeConfig(location_arguments),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      api_version='v1beta',
      disable_auto_completers=False,
  )


def GetRemoteProfileResourceSpec(location_arguments):
  return concepts.ResourceSpec(
      'networkconnectivity.projects.locations.remoteTransportProfiles',
      resource_name='remoteProfile',
      remoteTransportProfilesId=RemoteProfileAttributeConfig(),
      locationsId=LocationAttributeConfig(location_arguments),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      api_version='v1beta',
      disable_auto_completers=False,
  )


def GetResourceLocationArguments(resource_location_type):
  mapping = {
      ResourceLocationType.GLOBAL_ONLY: [GLOBAL_ARGUMENT],
      ResourceLocationType.REGION_ONLY: [REGION_ARGUMENT],
      ResourceLocationType.REGION_AND_GLOBAL: [
          GLOBAL_ARGUMENT,
          REGION_ARGUMENT,
      ],
  }
  return mapping[resource_location_type]


@enum.unique
class ResourceLocationType(enum.Enum):
  """Type of locations supported by a resource."""
  GLOBAL_ONLY = enum.auto()
  REGION_ONLY = enum.auto()
  REGION_AND_GLOBAL = enum.auto()


def AddSpokeResourceArg(
    parser, verb, resource_location_type=ResourceLocationType.REGION_AND_GLOBAL
):
  """Add a resource argument for a spoke.

  NOTE: Must be used only if it's the only resource arg in the command.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    resource_location_type: ResourceLocationType, the type of locations
      supported by the resource.
  """
  location_arguments = GetResourceLocationArguments(resource_location_type)
  presentation_spec = presentation_specs.ResourcePresentationSpec(
      name='spoke',
      concept_spec=GetSpokeResourceSpec(location_arguments),
      required=True,
      flag_name_overrides={'location': ''},
      group_help='Name of the spoke {}.'.format(verb),
  )
  concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)


def AddRegionResourceArg(
    parser, verb, resource_location_type=ResourceLocationType.REGION_AND_GLOBAL
):
  """Add a resource argument for a region.

  NOTE: Must be used only if it's the only resource arg in the command.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    resource_location_type: ResourceLocationType, the type of locations
      supported by the resource.
  """

  location_arguments = GetResourceLocationArguments(resource_location_type)
  presentation_spec = presentation_specs.ResourcePresentationSpec(
      name='region',
      concept_spec=GetRegionResourceSpec(location_arguments),
      required=True,
      flag_name_overrides={'location': ''},
      group_help='The region of the spokes {}.'.format(verb),
  )
  concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)


def AddPscGroup(parser):
  """Add a group which contains the PSC-related arguments to the given parser."""
  psc_group = parser.add_group(required=False, mutex=True)
  AddExportPscFlag(psc_group)
  psc_gapi_group = psc_group.add_group(required=False, mutex=False, hidden=True)
  AddExportPscPublishedServicesAndRegionalGoogleApisFlag(psc_gapi_group)
  AddExportPscGlobalGoogleApisFlag(psc_gapi_group)


def AddExportPscFlag(parser):
  """Add the --export-psc flag to the given parser."""
  parser.add_argument(
      '--export-psc',
      action=arg_parsers.StoreTrueFalseAction,
      required=False,
      help="""Whether Private Service Connect propagation is enabled for the hub.""",
  )


def AddExportPscPublishedServicesAndRegionalGoogleApisFlag(parser):
  """Add the --export-psc-published-services-and-regional-google-apis flag to the given parser."""
  parser.add_argument(
      '--export-psc-published-services-and-regional-google-apis',
      action=arg_parsers.StoreTrueFalseAction,
      required=False,
      hidden=True,
      help="""Whether propagation for Private Service Connect for published services and regional Google APIs is enabled for the hub.""",
  )


def AddExportPscGlobalGoogleApisFlag(parser):
  """Add the --export-psc-global-google-apis flag to the given parser."""
  parser.add_argument(
      '--export-psc-global-google-apis',
      action=arg_parsers.StoreTrueFalseAction,
      required=False,
      hidden=True,
      help="""Whether propagation for Private Service Connect for global Google APIs is enabled for the hub.""",
  )


def AddActivationKeyFlag(parser, help_text):
  """Adds the --activation-key flag to the given parser."""
  parser.add_argument('--activation-key', required=False, help=help_text)


def AddRemoteAccountIdFlag(parser, help_text):
  """Adds the --remote-account-id flag to the given parser."""
  parser.add_argument('--remote-account-id', required=False, help=help_text)


def AddProfileFlag(parser, help_text):
  """Adds the --remote_profile flag to the given parser."""
  parser.add_argument('--remote-profile', required=False, help=help_text)


def AddBandwidthFlag(parser, help_text):
  """Adds the --bandwidth flag to the given parser."""
  parser.add_argument('--bandwidth', required=True, help=help_text)


def AddAdvertisedRoutesFlag(parser, help_text, required=True):
  """Adds the --advertised-routes flag to the given parser."""
  parser.add_argument('--advertised-routes', required=required, help=help_text)


def AddEnableAdminFlag(parser, help_text):
  """Adds the --enable-admin flag to the given parser."""
  parser.add_argument(
      '--enable-admin',
      action=arg_parsers.StoreTrueFalseAction,
      required=False,
      help=help_text,
  )


def AddStackTypeFlag(parser, help_text):
  """Adds the --stack-type flag to the given parser."""
  parser.add_argument(
      '--stack-type',
      required=False,
      help=help_text,
      default='IPV4_ONLY',
  )


def AddTransportResourceArg(parser, desc):
  """Add a resource argument for a transport.

  Args:
    parser: the parser for the command.
    desc: the string to describe the resource, such as 'to create'.
  """

  location_arguments = GetResourceLocationArguments(
      ResourceLocationType.REGION_ONLY
  )
  presentation_spec = presentation_specs.ResourcePresentationSpec(
      name='transport',
      concept_spec=GetTransportResourceSpec(location_arguments),
      required=True,
      group_help='Name of the transport {}.'.format(desc),
  )
  concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)


def AddRemoteProfileResourceArg(parser, desc):
  """Add a resource argument for a remote transport profile."""
  location_arguments = GetResourceLocationArguments(
      ResourceLocationType.REGION_ONLY
  )
  presentation_spec = presentation_specs.ResourcePresentationSpec(
      name='remote_profile',
      concept_spec=GetRemoteProfileResourceSpec(location_arguments),
      required=True,
      group_help='Name of the remote transport profile {}.'.format(desc),
  )
  concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing Instances."""


from googlecloudsdk.api_lib.run import global_methods
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.run import commands
from googlecloudsdk.command_lib.run import connection_context
from googlecloudsdk.command_lib.run import exceptions
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.command_lib.run import resource_args
from googlecloudsdk.command_lib.run import serverless_operations
from googlecloudsdk.command_lib.run.printers import instance_printer
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core import properties
from googlecloudsdk.core.resource import resource_printer


@base.RegionalEndpointsSupported
class List(commands.List):
  """List instances."""

  detailed_help = {
      'DESCRIPTION': """
          {description}
          """,
      'EXAMPLES': """
          To list all instances in all regions:

              $ {command}
         """,
  }

  @classmethod
  def CommonArgs(cls, parser):
    namespace_presentation = presentation_specs.ResourcePresentationSpec(
        '--namespace',
        resource_args.GetNamespaceResourceSpec(),
        'Namespace to list instances in.',
        required=True,
        prefixes=False,
        hidden=True,
    )
    concept_parsers.ConceptParser([namespace_presentation]).AddToParser(parser)
    parser.display_info.AddFormat(
        'table('
        f'{instance_printer.StatusColorFormat()},'
        'name:label=INSTANCE,'
        'region:label=REGION,'
        'creation_timestamp.date("%Y-%m-%d %H:%M:%S %Z"):label="CREATED AT",'
        f'author:label="CREATED BY"):({commands.SATISFIES_PZS_ALIAS})'
    )
    parser.display_info.AddUriFunc(cls._GetResourceUri)
    resource_printer.RegisterFormatter(
        instance_printer.INSTANCE_PRINTER_FORMAT,
        instance_printer.InstancePrinter,
    )

  @classmethod
  def Args(cls, parser):
    cls.CommonArgs(parser)

  def Run(self, args):
    """List instances."""
    if not args.IsSpecified('region'):
      endpoint_mode = properties.VALUES.regional.endpoint_mode.Get()
      if endpoint_mode == properties.VALUES.regional.REGIONAL:
        raise exceptions.ArgumentError(
            'You must specify a region using the `--region` flag when '
            'regional endpoints are enabled.'
        )
      client = global_methods.GetServerlessClientInstance()
      self.SetPartialApiEndpoint(client.url)
      namespace_ref = args.CONCEPTS.namespace.Parse()  # Error if no proj.
      # Don't consider region property here, we'll default to all regions
      return commands.SortByName(
          global_methods.ListInstances(client, namespace_ref)
      )
    else:
      namespace_ref = args.CONCEPTS.namespace.Parse()
      conn_context = connection_context.GetConnectionContext(
          args, flags.Product.RUN, self.ReleaseTrack()
      )
      with serverless_operations.Connect(conn_context) as client:
        self.SetCompleteApiEndpoint(conn_context.endpoint)
        return commands.SortByName(client.ListInstances(namespace_ref))



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a new AlloyDB endpoint."""

from googlecloudsdk.api_lib.alloydb import api_util
from googlecloudsdk.api_lib.alloydb import endpoint_operations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.alloydb import endpoint_helper
from googlecloudsdk.command_lib.alloydb import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA
)
@base.DefaultUniverseOnly
class Create(base.CreateCommand):
  """Creates a new AlloyDB endpoint."""

  detailed_help = {
      'DESCRIPTION': '{description}',
      'EXAMPLES': """\
          To create a new write endpoint for an instance, run:

            $ {command} my-endpoint --region=us-central1 --endpoint-type=WRITE_ENDPOINT --target-instances=projects/my-project/locations/us-central1/clusters/my-cluster/instances/my-instance
          """
  }

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs
    """
    base.ASYNC_FLAG.AddToParser(parser)
    flags.AddEndpoint(parser)
    flags.AddRegion(parser)
    flags.AddTargetInstances(parser)
    flags.AddEndpointType(parser)

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    client = api_util.AlloyDBClient(self.ReleaseTrack())
    alloydb_client = client.alloydb_client
    alloydb_messages = client.alloydb_messages
    location_ref = client.resource_parser.Create(
        'alloydb.projects.locations',
        projectsId=properties.VALUES.core.project.GetOrFail,
        locationsId=args.region,
    )

    req = endpoint_helper.ConstructCreateRequestFromArgs(
        alloydb_messages, location_ref, args
    )
    op = alloydb_client.projects_locations_endpoints.Create(req)
    op_ref = resources.REGISTRY.ParseRelativeName(
        op.name, collection='alloydb.projects.locations.operations'
    )
    log.status.Print('Operation ID: {}'.format(op_ref.Name()))
    if not args.async_:
      endpoint_operations.Await(
          op_ref, 'Creating endpoint', self.ReleaseTrack()
      )
    return op



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cancel build command."""


from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.builds import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


@base.UniverseCompatible
class Cancel(base.Command):
  """Cancel an ongoing build."""

  detailed_help = {
      'DESCRIPTION':
          'Cancel an ongoing build.',
      'EXAMPLES': ("""
            To cancel a build `123-456-789`:

                $ {command} '123-456-789'

            You may also cancel multiple builds at the same time:

                $ {command} '123-456-789', '987-654-321'
            """),
  }

  @staticmethod
  def Args(parser):
    flags.AddRegionFlag(parser)
    parser.add_argument(
        'builds',
        completer=flags.BuildsCompleter,
        nargs='+',  # Accept multiple builds.
        help='IDs of builds to cancel')
    parser.display_info.AddFormat(None)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """
    build_region = (
        args.region
        or properties.VALUES.builds.region.Get()
        or cloudbuild_util.DEFAULT_REGION
    )

    client = cloudbuild_util.GetClientInstance()
    messages = cloudbuild_util.GetMessagesModule()

    cancelled = []
    for build in args.builds:
      build_ref = resources.REGISTRY.Parse(
          build,
          params={
              'projectsId': properties.VALUES.core.project.GetOrFail,
              'locationsId': build_region,
              'buildsId': build,
          },
          collection='cloudbuild.projects.locations.builds')
      cancelled_build = client.projects_locations_builds.Cancel(
          messages.CancelBuildRequest(
              name=build_ref.RelativeName(),
              projectId=build_ref.projectsId,
              id=build_ref.buildsId,
          ))
      log.status.write('Cancelled [{r}].\n'.format(r=str(build_ref)))
      cancelled.append(cancelled_build)
    return cancelled



# -*- coding: utf-8 -*- # Lint as: python3
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Compiles an Apigee template into a Proxy."""

import copy
from typing import Any, Dict, List, TypeVar, Callable, Optional, Union
from googlecloudsdk.command_lib.apigee.aft import models


_TypeToRewrite = TypeVar("_TypeToRewrite")
_FieldPath = List[Union[str, int]]


def _rewritten(
    dicted_data: Any,
    type_to_rewrite: type[_TypeToRewrite],
    rewriter: Callable[[_FieldPath, _TypeToRewrite], _TypeToRewrite],
    ancestor_path: Optional[_FieldPath] = None,
) -> Any:
  """Returns a copy of the given data structure with rewritten values.

  Args:
    dicted_data: The data to rewrite. If a Dict-like or List-like object, it
      will be traversed. Otherwise, it will be either rewritten (if of type
      type_to_rewrite) or copied as-is (if not of type type_to_rewrite).
    type_to_rewrite: The type to rewrite.
    rewriter: Called on each value of type type_to_rewrite with the value's
      full path within dicted_data; its return value will replace the original
      value.
    ancestor_path: If dicted_data is a nested object, this is the path to
      it from the top of the data structure.

  Returns:
    A copy of dicted_data with the specified values rewritten.
  """
  path: _FieldPath = ancestor_path or []
  if isinstance(dicted_data, type_to_rewrite):
    return rewriter(path, dicted_data)

  if isinstance(dicted_data, dict):
    rewritten = {}
    for key in dicted_data:
      value = dicted_data[key]
      child_path = path + [key]
      rewritten[key] = _rewritten(value, type_to_rewrite, rewriter, child_path)
    return rewritten

  if isinstance(dicted_data, list):
    return [
        _rewritten(e, type_to_rewrite, rewriter, path + [idx])
        for idx, e in enumerate(dicted_data)
    ]

  return copy.deepcopy(dicted_data)


def _make_replacer(
    needle: str, replacement: str
) -> Callable[[_FieldPath, str], str]:
  """Returns a function that replaces a string with another string.

  Args:
    needle: The string to replace.
    replacement: The string to replace it with.

  Returns:
    A function that takes a string and returns a copy of it where all
    occurrences of needle are replaced with replacement.
  """
  return lambda fp, x: x.replace(needle, replacement)


class ApigeeCompiler:
  """Compiles an Apigee template into a Proxy."""

  def compile(
      self,
      template: models.Template,
      features: List[models.Feature],
      parameters: Optional[Dict[str, str]] = None,
  ) -> models.Proxy:
    """Compiles an Apigee template into a Proxy.

    Args:
      template: The Apigee template to compile.
      features: The list of Apigee features to use. It's the caller's
        responsibility to look up the appropriate feature files referenced by
        the template's template.features field.
      parameters: The list of parameters to use.

    Returns:
      The compiled Apigee Proxy.
    """
    if parameters is None:
      parameters = {}

    # Convert template to dict for easier manipulation
    template_dict = models.to_dict(template)

    # Validate the required top-level schema fields (missing-first) before
    # composing. `template_dict` is camelCase, so read the camelCase keys with
    # a default so a missing field is reported cleanly by the validator rather
    # than raising a KeyError.
    models.validate_gateway(template_dict.get("gateway", ""))
    models.validate_schema_version(template_dict.get("schemaVersion", ""))

    proxy_dict = {
        # gateway/schemaVersion are authoring metadata carried onto the
        # compiled Proxy. compile() rebuilds the Proxy from proxy_dict via
        # models.from_dict below, so these keys are read back (and re-validated
        # by Proxy.__post_init__). They are dropped from the emitted bundle.
        "gateway": template_dict.get("gateway", ""),
        "schemaVersion": template_dict.get("schemaVersion", ""),
        "name": template_dict["name"],
        "description": template_dict["description"],
        "parameters": template_dict["parameters"],
        "priority": template_dict.get("priority"),
        "tests": template_dict.get("tests"),
        "endpoints": copy.deepcopy(template_dict.get("endpoints", [])),
        "policies": copy.deepcopy(template_dict.get("policies", [])),
        "targets": copy.deepcopy(template_dict.get("targets", [])),
        "resources": copy.deepcopy(template_dict.get("resources", [])),
    }

    proxy_dict = self.proxy_apply_features(proxy_dict, features, parameters)

    # Convert back to Proxy dataclass.
    return models.from_dict(models.Proxy, proxy_dict)

  def proxy_apply_features(
      self,
      proxy_dict: Dict[str, Any],
      features: List[models.Feature],
      parameters: Optional[Dict[str, str]] = None,
  ) -> Dict[str, Any]:
    """Applies features to a proxy dictionary in priority order."""
    if parameters is None:
      parameters = {}

    # Sort features by priority
    features = sorted(features, key=(
        lambda x: x.priority if x.priority is not None else 100
    ))

    # First apply features with targets & endpoints
    for feature in features:
      if feature.endpoints or feature.targets:
        proxy_dict = self.proxy_apply_feature(proxy_dict, feature, parameters)

    # Now apply features with just policies
    for feature in features:
      if not feature.endpoints and not feature.targets:
        proxy_dict = self.proxy_apply_feature(proxy_dict, feature, parameters)

    return proxy_dict

  def proxy_apply_feature(
      self,
      proxy_dict: Dict[str, Any],
      feature: models.Feature,
      parameters: Optional[Dict[str, str]] = None,
  ) -> Dict[str, Any]:
    """Applies a single feature to a proxy dictionary."""
    if parameters is None:
      parameters = {}

    feature_dict = models.to_dict(feature)

    # Replace parameters in feature
    feature_dict = self.feature_replace_parameters(
        feature_dict, proxy_dict["parameters"], parameters
    )

    # Namespace policies and resources. Use UID if available, otherwise just
    # the feature name.
    prefix = feature_dict.get("uid") or feature_dict["name"]

    namespaced_policy_names = {}
    for policy in feature_dict.get("policies", []):
      original_name = policy["name"]
      new_name = f"{prefix}-{original_name}"
      namespaced_policy_names[original_name] = new_name

    def policy_namespacer(fp: _FieldPath, x: str) -> str:
      if fp and fp[-1] == "name" and x in namespaced_policy_names:
        return namespaced_policy_names[x]
      if x.endswith(".") and x[:-1] in namespaced_policy_names:
        return namespaced_policy_names[x[:-1]] + "."
      return x

    feature_dict = _rewritten(feature_dict, str, policy_namespacer)

    namespaced_resource_names = {}
    namespaced_resource_references = {}
    for resource in feature_dict.get("resources", []):
      original_name = resource["name"]
      new_name = f"{prefix}-{original_name}"
      namespaced_resource_names[original_name] = new_name
      if original_name.endswith(".properties"):
        prop_name = original_name.replace(".properties", "")
        new_prop_name = new_name.replace(".properties", "")
        namespaced_resource_references[f"propertyset.{prop_name}."] = (
            f"propertyset.{new_prop_name}."
        )
      else:
        namespaced_resource_references[f"://{original_name}"] = (
            f"://{new_name}"
        )

    def resource_namespacer(fp: _FieldPath, x: str) -> str:
      if fp and fp[-1] == "name" and x in namespaced_resource_names:
        return namespaced_resource_names[x]
      for old_ref, new_ref in namespaced_resource_references.items():
        x = x.replace(old_ref, new_ref)
      return x

    feature_dict = _rewritten(feature_dict, str, resource_namespacer)

    # Merge flows
    # In TS, it merged defaultEndpoint flows into all proxy endpoints.
    if feature_dict.get("defaultEndpoint"):
      for endpoint in proxy_dict["endpoints"]:
        for feature_flow in feature_dict["defaultEndpoint"].get("flows", []):
          for proxy_flow in endpoint.get("flows", []):
            if (
                proxy_flow.get("name") == feature_flow.get("name")
                and proxy_flow.get("mode") == feature_flow.get("mode")
                and proxy_flow.get("condition") == feature_flow.get("condition")
            ):
              proxy_flow["steps"].extend(feature_flow.get("steps", []))
              break
          else:  # no matching flow was found
            if "flows" not in endpoint:
              endpoint["flows"] = []
            endpoint["flows"].append(copy.deepcopy(feature_flow))

        if feature_dict["defaultEndpoint"].get("defaultFaultRule"):
          if endpoint.get("defaultFaultRule"):
            endpoint["defaultFaultRule"]["steps"].extend(
                feature_dict["defaultEndpoint"]["defaultFaultRule"].get(
                    "steps", []
                )
            )
          else:
            endpoint["defaultFaultRule"] = copy.deepcopy(
                feature_dict["defaultEndpoint"]["defaultFaultRule"]
            )

    # Non-default endpoints and targets
    if "endpoints" in feature_dict and feature_dict["endpoints"]:
      existing_endpoints = {
          endpoint["name"]: idx
          for idx, endpoint in enumerate(proxy_dict["endpoints"])
          if "name" in endpoint
      }
      for endpoint in feature_dict["endpoints"]:
        if endpoint["name"] in existing_endpoints:
          # Replace existing endpoint with the feature endpoint
          idx = existing_endpoints[endpoint["name"]]
          proxy_dict["endpoints"][idx] = endpoint
        else:
          if "name" in endpoint:
            existing_endpoints[endpoint["name"]] = len(proxy_dict["endpoints"])
          proxy_dict["endpoints"].append(endpoint)

    if "targets" in feature_dict and feature_dict["targets"]:
      existing_targets = {
          target["name"]: idx
          for idx, target in enumerate(proxy_dict["targets"])
          if "name" in target
      }
      for target in feature_dict["targets"]:
        if target["name"] in existing_targets:
          # Replace existing target with the feature target
          idx = existing_targets[target["name"]]
          proxy_dict["targets"][idx] = target
        else:
          if "name" in target:
            existing_targets[target["name"]] = len(proxy_dict["targets"])
          proxy_dict["targets"].append(target)

    # Append policies and resources
    if feature_dict.get("policies"):
      proxy_dict["policies"].extend(feature_dict["policies"])
    if feature_dict.get("resources"):
      proxy_dict["resources"].extend(feature_dict["resources"])

    return proxy_dict

  def feature_replace_parameters(
      self,
      feature_dict: Dict[str, Any],
      proxy_parameters: List[Dict[str, Any]],
      parameters: Dict[str, str],
  ) -> Dict[str, Any]:
    """Applies parameters to a feature dictionary."""
    rewritten_feature = copy.deepcopy(feature_dict)

    for parameter in feature_dict.get("parameters", []):
      param_value = parameter.get("default", "")
      param_name = parameter.get("name", "")
      uid = feature_dict.get("uid")
      proxy_param_key = (
          f"{feature_dict['name']}.{uid}.{param_name}" if uid else param_name
      )

      # Find in proxy parameters
      proxy_param = next(
          (p for p in proxy_parameters if p["name"] == proxy_param_key), None
      )
      if proxy_param and proxy_param.get("default"):
        param_value = proxy_param["default"]

      if proxy_param_key in parameters:
        param_value = parameters[proxy_param_key]
      elif param_name in parameters:
        param_value = parameters[param_name]

      if parameter.get("maps") and param_value in parameter["maps"]:
        param_value = parameter["maps"][param_value]

      if parameter.get("paths"):
        # This is a JSONPath parameter. The intended behavior for such
        # parameters is to rewrite the values in the feature that are specified
        # specified by the JSONPath expressions.
        #
        # However, the Google Cloud SDK includes no Python library for JSONPath,
        # so such parameters won't be supported until feature processing is
        # moved to the server side.
        error_msg = (
            f"Parameter {param_name} uses JSONPath, which is not yet supported."
        )
        raise NotImplementedError(error_msg)
      else:
        # When JSONPath is not used, the parameter is applied by simple string
        # replacement: all strings in the feature that contain {param_name} have
        # that macro replaced with param_value.
        replace_key = "{" + param_name + "}"
        rewriter = _make_replacer(replace_key, param_value)
        rewritten_feature = _rewritten(rewritten_feature, str, rewriter)

    return rewritten_feature
